//****************************************************************
// DESENHOS.CPP
//
// Isabel H. Manssour e Márcio Pinho
// Disciplina: Programação 2D
// Especialização em Desenvolvimento de Jogos Digitais
//
// Este módulo realiza os desenhos dos objeto em coordenadas do
// sistema de referêncai do objeto, ou seja, SEM as transformacoes
// geometricas que definem a posicao, a rotacao e a escala de
// uma instancia
// Via de regra, eh neste fonte que devem ser colocadas os comandos
// OpenGL que fazem os desenhos
// **********************************************************************

// Para uso no Xcode:
// Abra o menu Product -> Scheme -> Edit Scheme -> Use custom working directory
// Selecione a pasta onde voce descompactou o ZIP que continha este arquivo.


#include <iostream>
#include <cmath>
#include <ctime>
#include <fstream>


using namespace std;

#ifdef WIN32
#include <windows.h>
#include <glut.h>
#else
#include <sys/time.h>
#endif

#ifdef __APPLE__
#include <GLUT/glut.h>
#endif

#ifdef __linux__
#include <glut.h>
#endif

#include "Desenhos.h"


static GLfloat angHelice=0;
//////////////////////////////////////////////////////////////////////////////////////////////////////
// Funo que desenha o contorno de um crculo a partir do raio e o centro recebidos por parmetro.
void DesenhaContornoCirculo(GLfloat raio, GLfloat centrox, GLfloat centroy)
{
    GLfloat x, y;
    glPointSize(2);
    for (GLfloat t=0; t<(6.28); t+=0.1) // de 0 a 360 graus (valor em radiano)
    {
        x = cos(t) * raio + centrox;
        y = sin(t) * raio + centroy;
        glBegin(GL_POINTS);
        glVertex2f(x,y);
        glEnd();
    }
}


//////////////////////////////////////////////////////////////////////////////////////////////////////
// Funo que desenha um crculo preenchido com a cor corrente a partir do raio e do centro recebidos
// por parmetro.
//
void DesenhaCirculo(GLfloat raio, GLfloat centrox, GLfloat centroy)
{
    GLfloat x, y;
    glLineWidth(3);
    for (GLfloat t=0; t<(6.28); t+=0.1) // de 0 a 360 graus (valor em radiano)
    {
        x = cos(t) * raio + centrox;
        y = sin(t) * raio + centroy;
        glBegin(GL_LINES);
        glVertex2f(centrox,centroy);
        glVertex2f(x,y);
        glEnd();
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Desenha um texto na janela GLUT.
//
void DesenhaTexto(char *string, GLfloat posX, GLfloat posY)
{
    glPushMatrix();
        // Posio no universo onde o texto ser colocado
        glRasterPos2f(posY,posY);
        // Exibe caracter a caracter
        while (*string)
            glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24,*string++);
    glPopMatrix();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////
// Funcao responsvel pelo desenho de um carro.
//
void DesenhaCarro()
{
    // Desenha a corroceria do carro com a cor azul
    glColor3f(0.0f, 0.0f, 1.0f);
    glBegin(GL_POLYGON);
        glVertex2i(  0, -11);
        glVertex2i(-82, -11);
        glVertex2i(-80,  11);
        glVertex2i(-67,  22);
        glVertex2i(-36,  26);
        glVertex2i(-26,  58);
        glVertex2i(-13,  72);
        glVertex2i(  1,  77);
        glVertex2i( 17,  72);
        glVertex2i( 33,  55);
        glVertex2i( 37,  31);
        glVertex2i( 61,  28);
        glVertex2i( 77,  20);
        glVertex2i( 87,   4);
        glVertex2i( 80, -11);
    glEnd();

    // Desenha o vidro dianteiro do carro com a cor cinza
    glColor3f(0.4f, 0.4f, 0.4f);
    glBegin(GL_POLYGON);
        glVertex2i(  1,  77);
        glVertex2i( 17,  72);
        glVertex2i( 33,  55);
        glVertex2i( 37,  31);
        glVertex2i(  1,  33);
    glEnd();

    // Desenha os pneus com a cor preta
    glColor3f(0.0f, 0.0f, 0.0f);
    DesenhaCirculo(15.0f, -35.0f,-11.0f);
    DesenhaCirculo(15.0f, 35.0f,-11.0f);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////
// Funcao responsvel pelo desenho de um helicoptero.
//
void DesenhaHelicoptero()
{
    // Desenha a corroceria do helicptero com a cor cinza
    glColor3f(0.4f, 0.4f, 0.4f);
    glBegin(GL_POLYGON);
        glVertex2i(-148, -6);
        glVertex2i(-148, 58); // HELICE TRASEIRA
        glVertex2i(-134, 48);
        glVertex2i(-126, 32);
        glVertex2i(-118, 12);
        glVertex2i(-118, -6);
        glEnd();
    glBegin(GL_POLYGON);
        glVertex2i(-118, 12);
        glVertex2i(-20, 22);
        glVertex2i(-20, -6);
        glVertex2i(-118, -6);
    glEnd();
        glBegin(GL_POLYGON);
        glVertex2i(-20, 22);
        glVertex2i(-2, 76);
        glVertex2i(20, 120);
        glVertex2i(74, 140);
        glVertex2i(124, 110);
        glVertex2i(148, 52);
        glVertex2i(134, -40);
        glVertex2i(8, -40);
        glVertex2i(-20, -6);
    glEnd();

    // Desenha as linhas da base e do suporte da hlice com a cor cinza
    glLineWidth(3);
    glBegin(GL_LINES);
        glVertex2i(46, -40);
        glVertex2i(24, -92);
        glVertex2i(96, -40);
        glVertex2i(118, -92);
        glVertex2i(74, 140);
        glVertex2i(74, 168); // HLICE SUPERIOR
        glEnd();
    glLineWidth(1);

    // Desenha o vidro dianteiro do helicptero com a cor cinza claro
    glColor3f(0.8f, 0.8f, 0.8f);
    glBegin(GL_POLYGON);
        glVertex2i(58, 110);
        glVertex2i(60, 68);
        glVertex2i(90, 46);
        glVertex2i(148, 52);
        glVertex2i(124, 110);
        glVertex2i(74, 140);
    glEnd();
    // Desenha as hlices

    // Incrementa o ângulo de rotação da hélice
    angHelice+=10;
    if (angHelice == 360)
        angHelice = 0;
    glPushMatrix();
    {
        glTranslatef(74, 168, 0); // HLICE SUPERIOR
        glRotatef(angHelice,0,0,1);
        glLineWidth(2);
        glColor3f(0.4f, 0.4f, 0.4f);
        glBegin(GL_LINES);
        glVertex2i(-80,0);
        glVertex2i( 80,0);
        glEnd();
    }
    glPopMatrix();
    glPushMatrix();
    {
        glTranslatef(-148, 58, 0); // HLICE TRASEIRA
        glRotatef(angHelice,0,0,1);
        glColor3f(0.4f, 0.4f, 0.4f);
        glBegin(GL_LINES);
        glVertex2i(-40,0);
        glVertex2i( 40,0);
        glEnd();
    }
    glPopMatrix();
    glLineWidth(1);

}


