// **********************************************************************
// PUCRS/Escola Politécnica
// COMPUTAÇÃO GRÁFICA
//
// Programa basico para criar aplicacoes 2D em OpenGL
//
// Marcio Sarroglia Pinho / Isabel Harb Manssour
// pinho@pucrs.br
// **********************************************************************

// Para uso no Xcode:
// Abra o menu Product -> Scheme -> Edit Scheme -> Use custom working directory
// Selecione a pasta onde voce descompactou o ZIP que continha este arquivo.

#include <iostream>
#include <cmath>
#include <ctime>
#include <fstream>

using namespace std;

#ifdef WIN32
#include <windows.h>
#include <glut.h>
#else
#include <sys/time.h>
#endif

#ifdef __APPLE__
#include <GLUT/glut.h>
#endif

#ifdef __linux__
#include <glut.h>
#endif

#include "Ponto.h"
#include "ListaDeCoresRGB.h"

#include "Desenhos.h"
#include "Tools.h"
#include "Objeto.h"

Objeto Cenario[10];

// Limites lógicos da área de desenho
Ponto Min, Max;

// Variáveis logicas
bool fimJogo, imprimeCoordenadas, tentaPegar;

// Variáveis que guardam os valores de translação e a direção do helicóptero manipulado pelo usuário
GLfloat txHelicoptero, tyHelicoptero;

// Variável que guarda o quanto o quadrado será deslocado cada vez que o usuário pressiona uma tecla de seta
GLfloat delta;

int QTD_OBJETOS;
int ObjetoAtivo;

// **********************************************************************
// Imprime o texto S na posicao (x,y), com a cor 'cor'
// **********************************************************************
void printString(string S, int x, int y, int cor)
{
    defineCor(cor);
    glRasterPos3f(x, y, 0); //define posicao na tela
    for (int i = 0; i < S.length(); i++)
    {
        // GLUT_BITMAP_HELVETICA_10
        // GLUT_BITMAP_TIMES_ROMAN_24
        // GLUT_BITMAP_HELVETICA_18
        glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24, S[i]);
    }
}
#include <sstream>
using std::stringstream;

// **********************************************************************
// Converte um nro em string
// **********************************************************************
string toString(float f)
{
    stringstream S;
    S << f;
    return S.str();
}
// **********************************************************************
// Imprime as coordenadas do ponto P na posicao (x,y), com a cor 'cor'
// **********************************************************************
void ImprimePonto(Ponto P, int x, int y, int cor)
{
    string S;
    S = "( " + toString(P.x) + ", " + toString(P.y) + ")" ;
    printString(S, x, y, cor);
}

// **********************************************************************
//
// **********************************************************************
void CarregaModelos () {

    QTD_OBJETOS = 3;
    ObjetoAtivo = 0;

    for(int i=1; i<QTD_OBJETOS; i++) {
        Cenario[i].Translate(i*600,i*300,0);
        Cenario[i].setTipo(2); // carro
        Cenario[i].setID(i);
    }
    Cenario[0].setTipo(1); // Helicoptero
    // Helicoptero.ImprimeMatriz("Apos o INIT");

    imprimeCoordenadas = false;
    tentaPegar = false;

}

// **********************************************************************
//
// **********************************************************************
void init()
{
    // Define a cor do fundo da tela 
    glClearColor(1.0f, 1.0f, 1.f, 1.0f);

    delta = 30; // defini o descolamento dos objetos a cada passo.
    CarregaModelos();

    Min = Ponto(-1600, -900);
    Max = Ponto(1600,900);
}

// **********************************************************************
//
// **********************************************************************
void DesenhaEixos()
{
    Ponto Meio;
    Meio.x = (Max.x+Min.x)/2;
    Meio.y = (Max.y+Min.y)/2;
    Meio.z = (Max.z+Min.z)/2;

    glBegin(GL_LINES);
    //  eixo horizontal
        glVertex2f(Min.x,Meio.y);
        glVertex2f(Max.x,Meio.y);
    //  eixo vertical
        glVertex2f(Meio.x,Min.y);
        glVertex2f(Meio.x,Max.y);
    glEnd();
}
void imprimeCoordenadasDosObjetos()
{
    GLfloat P1[] = {0,0,0};
    GLfloat P2[3];

    imprimeCoordenadas = false;
    printf("Posicoes do objetos:\n");
    for(int i=0; i<QTD_OBJETOS; i++) {
        printf("Objeto %d: ",i);
        Cenario[i].InstanciaPonto(P1,P2);
        for (int j=0; j<3; j++)
            printf("%7.2f ", P2[j]);
        printf("\n");
    }
    printf("======================================\n");
}
// **********************************************************************
// Função callback de redesenho da janela de visualização.
// **********************************************************************
//
void display(void) {

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// Limpa a janela
	glClear(GL_COLOR_BUFFER_BIT);
    
    DesenhaEixos();

	char msg[100];
	if (ObjetoAtivo==0)
		strcpy(msg, "Objeto Ativo: Helicoptero");
	else sprintf(msg, "Objeto Ativo: Carro %d",ObjetoAtivo);

	DesenhaTexto(msg, Min.x+100, Min.y+100);

	// Desenha o cenario todo
	for(int i=0; i<3; i++) {
		Cenario[i].Draw();
	}
	// Imprime as coordenadas do ponto de inserção de todos os objetos do cenário
	if (imprimeCoordenadas)
        imprimeCoordenadasDosObjetos();

	glutSwapBuffers();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////
// Função callback chamada quando o tamanho da janela é alterado.
//
// **********************************************************************
//  void reshape( int w, int h )
//  trata o redimensionamento da janela OpenGL
// **********************************************************************
void reshape( int w, int h )
{
    // Reset the coordinate system before modifying
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    // Define a area a ser ocupada pela area OpenGL dentro da Janela
    glViewport(0, 0, w, h);
    // Define os limites logicos da area OpenGL dentro da Janela
    
    glOrtho(Min.x,Max.x, Min.y,Max.y, 0,1);
    //glOrtho(0,10, 0,10, 0,1);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}
double AccumDeltaT=0;
// **********************************************************************
//
// **********************************************************************
void animate()
{
    double dt;
    dt = glutGet(GLUT_ELAPSED_TIME);
    AccumDeltaT += dt;
    
    if (AccumDeltaT > 1.0/30) // fixa a atualização da tela em 30
    {
        AccumDeltaT = 0;
        glutPostRedisplay();
    }
}


//////////////////////////////////////////////////////////////////////////////////////////////////////
// Função callback chamada para gerenciar eventos de teclas.
//
void keyboard (unsigned char key, int x, int y) {
	// define qual dos objetos deve ser ativado (de 0 até 9)
	// O objeto 0 eh o helicptero
	if (key >= '0' && key <= '9') { // ativa objetos de 1 a 9
		ObjetoAtivo = key - '0'; // converte para o indice do objeto ativo;
		if (ObjetoAtivo >= QTD_OBJETOS)
			ObjetoAtivo = 0; // caso haj algum erro no nro do objeto, ativa o helicoptero
		printf ("---------------------------------\nObjeto Ativado: %d\n", ObjetoAtivo );
	}
	switch (key) {

	case ' ': //espaco
		imprimeCoordenadas = true;
		break;

	case 'p': // faz o helicoptero pegar um dos objetos do cenario
	case 'P':
		Cenario[0].AdicionaFilho(&Cenario[1]);
		break;
	case 'l': // faz o helicoptero largar um dos objetos do cenario
	case 'L':
		Cenario[0].RemoveFilho();
		break;
	case 27:
		exit(1);
	}
	glutPostRedisplay();
}
// **********************************************************************
// Converte as coordenadas do ponto P de coordenadas de tela para
// coordenadas de universo (sistema de referência definido na glOrtho
// (ver função reshape)
// Este código é baseado em http://hamala.se/forums/viewtopic.php?t=20
// **********************************************************************
Ponto ConvertePonto(Ponto P)
{
    GLint viewport[4];
    GLdouble modelview[16],projection[16];
    GLfloat wx=P.x,wy,wz;
    GLdouble ox=0.0,oy=0.0,oz=0.0;
    glGetIntegerv(GL_VIEWPORT,viewport);
    P.y=viewport[3]-P.y;
    wy=P.y;
    glGetDoublev(GL_MODELVIEW_MATRIX,modelview);
    glGetDoublev(GL_PROJECTION_MATRIX,projection);
    glReadPixels(P.x,P.y,1,1,GL_DEPTH_COMPONENT,GL_FLOAT,&wz);
    gluUnProject(wx,wy,wz,modelview,projection,viewport,&ox,&oy,&oz);
    return Ponto(ox,oy,oz);
}
// **********************************************************************
// Função callback chamada para gerenciar eventos do mouse.
// **********************************************************************
void Mouse(int button,int state,int x,int y)
{
    //cout << "Mouse..." << endl;

    if(state==GLUT_DOWN && button==GLUT_LEFT_BUTTON)
    {
        cout << "Botao ESQ Desceu" << endl;
    }
    if(state==GLUT_UP && button==GLUT_LEFT_BUTTON)
    {
        cout << "Botao ESQ Subiu" << endl;
    }
    if(state==GLUT_UP)
      return;
    if(button==GLUT_RIGHT_BUTTON)
     return;
    
    Ponto PontoClicado;
    PontoClicado = ConvertePonto(Ponto(x,y,0));
   
}
// **********************************************************************
// Função callback chamada para gerenciar eventos de teclas especiais(F1,PgDn,...).
// **********************************************************************
void arrow_keys(int key, int x, int y) {
	if (key == GLUT_KEY_UP) { // desloca o objeto ativo para cima
		Cenario[ObjetoAtivo].Rotate(10,0,0,1);
	}
	if (key == GLUT_KEY_DOWN) { // desloca o objeto ativo para baixo
		Cenario[ObjetoAtivo].Rotate(-10,0,0,1);
	}
	if (key == GLUT_KEY_LEFT) { // desloca o objeto ativo para esquerda
		Cenario[ObjetoAtivo].Translate(-delta,0,0);
	}
	if (key == GLUT_KEY_RIGHT) { // desloca o objeto ativo para direita
		Cenario[ObjetoAtivo].Translate(delta,0,0);
	}
	//glutPostRedisplay();
}


// **********************************************************************
// Programa Principal.
// **********************************************************************
int main(int argc, char *argv[]) {

	glutInit(&argc, argv);

	// Define do modo de operação da GLUT
    glutInitDisplayMode (GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGB );

	// Especifica a posição inicial da janela GLUT
	glutInitWindowPosition(10,10);

	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(960,540);
    
    // Cria a janela na tela, definindo o nome da
    // que aparecera na barra de titulo da janela.
	glutCreateWindow("Hierarquia");
    
    // Chama a função responsável por fazer as inicializações
    init();


    // Define que o tratador de evento para
    // o redesenho da tela. A funcao "display"
    // ser· chamada automaticamente quando
    // for necess·rio redesenhar a janela
    glutDisplayFunc ( display );

    // Define que o tratador de evento para
    // o invalidação da tela. A funcao "display"
    // ser· chamada automaticamente sempre que a
    // máquina estiver ociosa (idle)
    glutIdleFunc(animate);

    // Define que o tratador de evento para
    // o redimensionamento da janela. A funcao "reshape"
    // ser· chamada automaticamente quando
    // o usu·rio alterar o tamanho da janela
    glutReshapeFunc ( reshape );

    // Define que o tratador de evento para
    // as teclas. A funcao "keyboard"
    // ser· chamada automaticamente sempre
    // o usu·rio pressionar uma tecla comum
    glutKeyboardFunc ( keyboard );

    // Define que o tratador de evento para
    // as teclas especiais(F1, F2,... ALT-A,
    // ALT-B, Teclas de Seta, ...).
    // A funcao "arrow_keys" ser· chamada
    // automaticamente sempre o usu·rio
    // pressionar uma tecla especial
    glutSpecialFunc ( arrow_keys );
    glutMouseFunc(Mouse);

    // inicia o tratamento dos eventos
    glutMainLoop ( );

	return 0;
}
