//****************************************************************
// Objeto.CPP
//
// Isabel H. Manssour e Marcio Pinho
// Disciplina: Programacao 2D
// Especializacao em Desenvolvimento de Jogos Digitais
//
// Este modulo define a estutura que armazena uma INSTANCIA de um
// objeto
//****************************************************************

#include <iostream>
#include <cmath>
#include <ctime>
#include <fstream>


using namespace std;

#ifdef WIN32
#include <windows.h>
#include <glut.h>
#else
#include <sys/time.h>
#endif

#ifdef __APPLE__
#include <GLUT/glut.h>
#endif

#ifdef __linux__
#include <glut.h>
#endif

#include "Desenhos.h"
#include "Objeto.h"
#include "Tools.h"

// **********************************************************************
//  Just for debugging...
// **********************************************************************
void Objeto::ImprimeMatriz(const char *msg) {
	printf("=============================================\n");
	printf("%s\n",msg);
	for(int i=0; i<4; i++) {
		for(int j=0; j<4; j++)
			printf("%7.3f ", M[i][j]);
		printf("\n");
	}
}

// **********************************************************************
// Metodo que instancia o ponto PontoIN no sistema de referencia da instancia
// fazndo a multiplicacao do ponto PontoIN pela Matriz de Transformacao (M) da
// instancia
// **********************************************************************
void Objeto::InstanciaPonto(GLfloat PontoIN[3], GLfloat PontoOUT[3]) {
	PontoOUT[0] = M [0][0] * PontoIN[0] + M[1][0] * PontoIN[1] + M[2][0] * PontoIN[2] + M[3][0];
	PontoOUT[1] = M [0][1] * PontoIN[0] + M[1][1] * PontoIN[1] + M[2][0] * PontoIN[2] + M[3][1];
	PontoOUT[2] = M [0][2] * PontoIN[0] + M[1][2] * PontoIN[1] + M[2][0] * PontoIN[2] + M[3][2];
}

// **********************************************************************
// Metodo que inicializa os parmetros default de um objeto, colocando-o
// como a translacao e a rotacao zeradas e com a escala em 1.
// **********************************************************************
 Objeto::Objeto() {
    criaIdentidade(M);
  	nroDeFilhos = 0;
	eh_filho = false;
	Filho = NULL;
}
// **********************************************************************
// Define um ID para o objeto
// **********************************************************************
void Objeto::setID(int i)
{
    id = i;
}
// **********************************************************************
// Define o tipo do objeto
// **********************************************************************
void Objeto::setTipo(int t) {
	tipo = t;
}
// **********************************************************************
// Metodos que aplicam as transformacoes geometricas
// **********************************************************************
void Objeto::Rotate(float ang, float x, float y, float z) {
	glPushMatrix();
	glLoadMatrixf(&M[0][0]);
	glRotatef(ang, x, y, z);
	glGetFloatv(GL_MODELVIEW_MATRIX, &M[0][0]);
	glPopMatrix();
}

void Objeto::Translate(float x, float y, float z) {
	glPushMatrix();
	glLoadMatrixf(&M[0][0]);
	glTranslatef(x++, y, z);
	glGetFloatv(GL_MODELVIEW_MATRIX, &M[0][0]);
	glPopMatrix();
}
void Objeto::Scale(float x, float y, float z) {
	glPushMatrix();
	glLoadMatrixf(&M[0][0]);
	glScalef(x, y, z);
	glGetFloatv(GL_MODELVIEW_MATRIX, &M[0][0]);
	glPopMatrix();
}

// **********************************************************************
//  Exercicio: este metodo devera ser alterado de forma a permitir o
//  desenho dos 'filhos' do objeto. Veja os slides da aula para entender
//  como fazer.
// **********************************************************************
void Objeto::Draw() {
	if (eh_filho) {
		printf("Metodo Draw: Voce esta tentando desenhar um FILHO antes de seu PAI\n");
        // Quem deve desenhar um objeto é 
		return; // Nao desenha se o objeto eh filho de algum outro
	}
	glPushMatrix();
	glMultMatrixf(&M[0][0]);
	switch(tipo) {
	case 1:
		DesenhaHelicoptero();
		if (!Filho) // se não tiver filhos, entao termina
			break;
		// libera o filho para desenho
		Filho->eh_filho = false;
		// Ativa o desenho dos Filhos
		Filho->Draw(); // alterar para desenhar todos os filhos
		// Bloqueia o filho para desenho
		Filho->eh_filho = true;
		break;
	case 2:
		DesenhaCarro();
		break;
	}
	glPopMatrix();
}
// **********************************************************************
//  Exercicio: este metodo devera' ser alterado de forma a permitir o
//  que um objeto passe a ter um 'filho' do objeto.
//  Veja os slides da aula para entender como fazer.
//  Ao adicionar um objeto como filho de outro, lembre-se que a partir
//  deste momento o objeto 'filho' nao deve mais ser desenhado
// **********************************************************************
void Objeto::AdicionaFilho(Objeto *filho) {
	
	if (Filho != NULL) return;
	// evita que a rotina de desenho exiba o objeto sem que isto
	// seja feito atraves de seu pai...
	filho->eh_filho = true;

	// adiciona o filho na lista do pai
	Filho = filho;
	Filho->Pai = this;

	// Aqui é preciso desfazer a transformação da matriz do pai
	//    MFilho = MFilho * Inv(MPai)
	// ************************************************
	
}

// **********************************************************************
//  Exercicio: este metodo devera ser alterado de forma a permitir o
//  que um objeto deixer de ser filho de outro.
//  Veja os slides da aula para entender como fazer.
//  Ao remover  um objeto como filho de outro, lembre-se que a partir
//  deste momento o objeto 'filho' nao deve mais ser desenhado
// **********************************************************************
void Objeto::RemoveFilho() {
	printf("Remove....");
	fflush(stdout);
	if(Filho == NULL) {
		printf("Nao tem filho.");
		fflush(stdout);
		return;
	}
	// Libere o objeto para que a rotina de desenho possa exibo-lo 
	// sem que isto seja feito atraves de seu pai...
	
// Aqui o filho deve passar a ter como a sua matriz de transformação a matriz dele próprio,
// multiplicada pela matriz do pai.
//	MFilho = MPai * MFilho

	// sinaliza que nao tem mais filho
	Filho = NULL;

}
